﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using BroadcastEmail.DataAccess.Custom;
using BroadcastEmail.DataAccess.Framework;
using Infragistics.WebUI.UltraWebNavigator;
using System.Web.Script.Serialization;

namespace BroadcastEmail.Distribution
{
    public partial class BusinessPlan : BasePage
    {
        #region Propeties
        public DataAccess.Distribution DistributionList
        {
            get
            {
                return SessionHelper.GetSessionValue<DataAccess.Distribution>(SessionKeys.DistributionList, SessionVariableAllowNull.Allow);
            }


            set
            {
                Session[SessionKeys.DistributionList] = value;
            }
        }

        #endregion


        #region "Message and Error"
        protected override Label MessageLabel
        {
            get { return MsgLabel; }
        }

        protected override Label ErrorLabel
        {
            get { return ErrLabel; }
        }

        #endregion

        protected void Page_Load(object sender, EventArgs e)
        {
            base.Page_Load(sender, e);

            if (!Page.IsPostBack)
            {
                InitializePage();
                PopulateBusinessPlanTreeRootNode();
            }

        }

        protected void Save_Click(object sender, EventArgs e)
        {
            int groupId = int.MinValue;

            try
            {

                groupId = SessionHelper.GetSessionValue<int>(SessionKeys.GroupId);

                if (DistributionList.IsNew)
                {
                    DistributionList.CreatedDateTime = DateTime.Now;
                    DistributionList.CreatedHIGRDN = WebHelper.HigRdn;
                }

                DistributionList.TypeId = 3;
                DistributionList.UpdatedDateTime = DateTime.Now;
                DistributionList.UpdatedHIGRDN = WebHelper.HigRdn;
                DistributionList.DistributionName = DistributionName.Text;
                DistributionList.IsActive = ActiveCheckBox.Checked;
                DistributionList.IsProof = ProofOnlyCheckBox.Checked;
                DistributionList.Comment = Comments.Text;
                DistributionList.EmployeesOnly = EmployeeOnlyCheckBox.Checked;
                DistributionList.ManagersOnly = ManagersOnlyCheckBox.Checked;
                DistributionList.ExcludeHartTemp = ExcludeHartTempsCheckBox.Checked;
                DistributionList.ExcludePartTime = ExcludePartTimeCheckBox.Checked;

                List<string> includeBusinessPlan = GetSelectBusinessPlanItems(IncludeList.Nodes);
                List<string> exludeBusinessPlan = GetSelectBusinessPlanItems(ExcludeList.Nodes);

                Business.Distribution.DistributionByBusinessPlan.Save(
                      DistributionList
                    , groupId
                    , includeBusinessPlan
                    , exludeBusinessPlan
                    , DistributionLocation.SelectedCounties
                    , DistributionLocation.SelectedStates
                    , DistributionLocation.SelectedCities
                    , DistributionTier.SelectedTiers);

                Session[SessionKeys.DisplayMessage] = string.Format("Distribution '{0}' saved", DistributionName.Text);

                Response.Redirect(string.Format("BusinessPlan.aspx?{0}={1}", QueryStringKeys.DistributionID, DistributionList.DistributionID), Constants.EndResponse);

            }
            catch (Exception saveEx)
            {
                DisplayError(saveEx.Message);
            }

        }

        protected void Cancel_Click(object sender, EventArgs e)
        {
            // Todo: Reset the form.
            InitializePage();
        }

        private void InitializePage()
        {
            DistributionName.Text = string.Empty; 

            ProofOnlyCheckBox.Checked = false;
            ActiveCheckBox.Checked = false;
            Comments.Text = string.Empty;
            ActiveCheckBox.Checked = true;
            EmployeeOnlyCheckBox.Checked = true;
            ManagersOnlyCheckBox.Checked = false;
            ExcludeHartTempsCheckBox.Checked = false;
            ExcludePartTimeCheckBox.Checked = false;

            string distID = QueryStringHelper.GetQueryStringValue(QueryStringKeys.DistributionID, QueryStringHelper.Options.NotRequired);
            int distributionID;

            if (int.TryParse(distID, out distributionID))
            {
                if (distributionID.Equals(int.MinValue))
                {
                    DistributionList = DataAccess.Distribution.CreateDistribution();
                }
                else
                {
                    DistributionList = DataAccess.Distribution.GetDistribution(distributionID);
                }
            }
            else
            {
                DistributionList = DataAccess.Distribution.CreateDistribution();
            }

            if (DistributionList != null)
            {
                PopulateDistributionInfo();
                PopulateIncludeAndExcludeList();
                PopudateEmployeeType();
                PopulateCount();
            }

            DistributionTier.Initialize();
            DistributionLocation.Initialize();
        }

        private void PopulateDistributionInfo()
        {
            this.
            DistributionName.Text = DistributionList.DistributionName;
            ProofOnlyCheckBox.Checked = DistributionList.IsProof;
            ActiveCheckBox.Checked = DistributionList.IsActive;

            //
            // ToDo: Not sure what do do about replace excisting.
            //

            Comments.Text = DistributionList.Comment;

        }

        private void PopulateIncludeAndExcludeList()
        {
            this.IncludeList.Nodes.Clear();
            this.ExcludeList.Nodes.Clear();

            if (DistributionList != null)
            {
                foreach (var item in DistributionList.DistributionQueryPlans)
                {
                    if (item.Query.IncludeExclude == true)
                    {
                        AddQueryPlanToList(item.Query, this.IncludeList);
                    }
                    else
                    {
                        AddQueryPlanToList(item.Query, this.ExcludeList);
                    }
                }
            }

        }

        private void PopulateLocations()
        {
            // ToDo: Delete this method
        }

        private void PopulateTier()
        {
            // ToDo: Delete this method
        }

        private void PopudateEmployeeType()
        {
            if (DistributionList != null)
            {
                ManagersOnlyCheckBox.Checked = DistributionList.ManagersOnly;
                EmployeeOnlyCheckBox.Checked = DistributionList.EmployeesOnly;
                ExcludeHartTempsCheckBox.Checked = DistributionList.ExcludeHartTemp;
                ExcludePartTimeCheckBox.Checked = DistributionList.ExcludePartTime;
            }
        }

        private void PopulateCount()
        {
            if (!DistributionList.DistributionID.Equals(int.MinValue))
            {
                DataAccess.Custom.ItemEmailAddress itemEmailAddress = new ItemEmailAddress();
                int count = itemEmailAddress.GetDistributionCount(DistributionList.DistributionID, DistributionList.IsProof);
                this.CountLabel.Text = string.Format("Count:  {0}", count);
                this.CountLabel.Visible = true;
            }
        }

        private void AddQueryPlanToList(DataAccess.QueryPlan query, UltraWebTree tree)
        {
            Node node = new Node();
            node.Text = query.Value;
            //
            // NOTE:  Store the existing QueryPlan info in the tag properity
            //        format:  QueryID, Name , Value, IncludeExclude, Active
            //        This is needed when the Distribuiton is saved.
            //
            DataAccess.Custom.BusinessPlan businessPlan = new BroadcastEmail.DataAccess.Custom.BusinessPlan();
            businessPlan.DisplayValue = query.DisplayName;
            businessPlan.KeyValue = query.Value;
            businessPlan.KeyName = query.Name;
            node.Text = query.DisplayName;
            node.Tag = (object)businessPlan.ToJson();
            tree.Nodes.Add(node);
        }

        private void PopulateGroupList(DropDownList groupList)
        {
            if (groupList != null)
            {
                groupList.Items.Clear();
                DataAccess.Framework.EntityList<DataAccess.Group> groups = DataAccess.Group.GetGroups();
                groupList.DataSource = groups;
                groupList.DataTextField = "GroupName";
                groupList.DataValueField = "GroupID";
                groupList.DataBind();
            }
        }


        // ToDo: I don't think this method is used so delete it.
        private List<string> GetSelectBusinessPlanItems(Nodes nodes)
        {
            List<string> list = new List<string>();


            // Test Code Only:
            foreach (var item in nodes)
            {
                Node itemNode = (Node)item;
                list.Add(itemNode.Tag.ToString());
            }

            return list;
        }

        #region "Business Plan Tree Views"

        private void PopulateBusinessPlanTreeRootNode()
        {
            Node roodNode = new Node();
            roodNode.Text = DataAccess.Custom.BusinessPlan.Level.HIG.ToString();
            roodNode.Tag = DataAccess.Custom.BusinessPlan.Level.HIG;
            BusinessPlanTree.Nodes.Add(roodNode);

            EntityListReadOnly<DataAccess.Custom.BusinessPlan> items = null;
            items = DataAccess.Custom.BusinessPlan.GetBusinessPlanByLevel(DataAccess.Custom.BusinessPlan.Level.HIG, "HIG");

            foreach (var item in items)
            {
                Node childNode = new Node();
                childNode.Text = item.DisplayValue;

                //
                // ToDo: Change this so that new object and existing object are the same (queryPlan objects)
                //       something like
                //
                //string tag = string.Format("QueryID:{0} Name:{1} Value:{2}", int.MinValue,item.KeyName,item.KeyValue);
                childNode.Tag = (object)item.ToJson();
                childNode.ShowExpand = true;
                BusinessPlanTree.Nodes[0].Nodes.Add(childNode);
                BusinessPlanTree.ExpandAll();
            }
        }


        protected void BusinessPlanTree_DemandLoad(object sender, WebTreeNodeEventArgs e)
        {

            EntityListReadOnly<DataAccess.Custom.BusinessPlan> items = null;
            DataAccess.Custom.BusinessPlan.Level level = (DataAccess.Custom.BusinessPlan.Level)e.Node.Level;

            JavaScriptSerializer serializer = new JavaScriptSerializer();
            DataAccess.Custom.BusinessPlan selectedNode = serializer.Deserialize<DataAccess.Custom.BusinessPlan>(e.Node.Tag.ToString());

            items = DataAccess.Custom.BusinessPlan.GetBusinessPlanByLevel(level, selectedNode.KeyValue);

            foreach (var item in items)
            {
                Node childNode = e.Node.Nodes.Add(item.DisplayValue, (object)item.ToJson());
                DataAccess.Custom.BusinessPlan.Level childLevel = (DataAccess.Custom.BusinessPlan.Level)childNode.Level;
                if (childLevel != DataAccess.Custom.BusinessPlan.Level.departmentnumber)
                {
                    childNode.ShowExpand = true;
                }
            }

            e.Node.Expanded = true;

        }

        #endregion



    }
}

